@@ -338,7 +338,7 @@ class Item:
 
     def __report(self, action, item):
         if self.__callback:
-            self.__callback((action, item))
+            self.__callback((action, item, None))
 
 
     def __calc_the_tag(self, tag):
@@ -574,7 +574,8 @@ class Feed:
         page_template = None, page_item_template = None, print_version_keywords = None, \
         try_use_print_version = True, delete_before_date = None, allow_scripts = False, \
         inline_items = False, cache_items = False, include = None, exclude = None, as_is = False,
-        content_kwargs = None, reformatter_url = None, next_page_keyword = None):
+        content_kwargs = None, reformatter_url = None, next_page_keyword = None, 
+        new_window = False, always_clear_cache = False):
         """ Constructor.
 
         url - feed url.
@@ -599,6 +600,7 @@ class Feed:
         tag name and its attributes.
         reformatter_url - format string (%s is replaced with url from item).
         next_page_keyword - the name of the link that leads to the next page of document if any
+        new_window - True if you want all links in the feed to open new browser window.
         Used to process html document through online services such as Skweezer.
         """
         self.__url = url
@@ -629,12 +631,14 @@ class Feed:
         self.__content_kwargs = content_kwargs
         self.__reformatter_url = reformatter_url
         self.__next_page_keyword = next_page_keyword
+        self.__new_window = new_window
+        self.__always_clear_cache = always_clear_cache
         self.__downloader = Downloader(callback)
 
 
-    def __report(self, action, item):
+    def __report(self, action, item, percent = None):
         if self.__callback:
-            self.__callback((action, item))
+            self.__callback((action, item, percent))
 
 
     def __unquote_special(self, str):
@@ -710,31 +714,32 @@ class Feed:
             downloader = self.__downloader)
         
 
-    def __add_item(self, div, item):
+    def __add_item(self, div, item, percent):
         """ Here Item is get included to feed after executing all dependant operations. """
         link = self.__get_item_link(item)
-        if self.__match_link(link) and not self.__outdated(item) and not self.__item_exists(div.parent, link):
-            title = self.__unquote_special(item.title.string.strip())
-            self.__report("processing", title)
-            item_html = BeautifulSoup(self.__page_item_template)
-            div.append(Comment(NavigableString(self.item_comment_prefix + link)))
-            div.append(item_html)
-            item_a = get_or_create_tag(item_html, ["a"])
-            item_a["href"] = link
-            item_a.append(NavigableString(title))
-            item_div = get_or_create_tag(item_html, ["div"])
-            if self.__inline_items:
-                feed_item = self.__create_feed_item(url = link)
-                [item_div.append(t) for t in feed_item.process()]
-            else:
-                feed_item = self.__create_feed_item(tag = self.__get_item_description(item), base = self._real_url)
-                [item_div.append(t) for t in feed_item.process()]
-                if self.__cache_items:
-                    path = os.path.join(self.__path, self.__cache_path)
-                    feed_item = self.__create_feed_item(url = link, path = path, cache_path = "")
-                    feed_item.process()
-                    item_a["href"] = os.path.join(self.__cache_path, feed_item.save(title))
-                    item_div.insert(0, Comment(NavigableString(Item.cached_comment_prefix + item_a["href"])))
+        title = self.__unquote_special(item.title.string.strip())
+        self.__report("processing", title, percent)
+        item_html = BeautifulSoup(self.__page_item_template)
+        div.append(Comment(NavigableString(self.item_comment_prefix + link)))
+        div.append(item_html)
+        item_a = get_or_create_tag(item_html, ["a"])
+        item_a["href"] = link
+        item_a.append(NavigableString(title))
+        item_div = get_or_create_tag(item_html, ["div"])
+        if self.__inline_items:
+            feed_item = self.__create_feed_item(url = link)
+            [item_div.append(t) for t in feed_item.process()]
+        else:
+            feed_item = self.__create_feed_item(tag = self.__get_item_description(item), base = self._real_url)
+            [item_div.append(t) for t in feed_item.process()]
+            if self.__cache_items:
+                path = os.path.join(self.__path, self.__cache_path)
+                feed_item = self.__create_feed_item(url = link, path = path, cache_path = "")
+                feed_item.process()
+                item_a["href"] = os.path.join(self.__cache_path, feed_item.save(title))
+                if self.__new_window:
+                    item_a["target"] = "_blank"
+                item_div.insert(0, Comment(NavigableString(Item.cached_comment_prefix + item_a["href"])))
 
 
     def __delete_div(self, div):
@@ -743,6 +748,12 @@ class Feed:
         div.extract()
 
 
+    def __delete_all_divs(self, html):
+        if html.body:
+            for div in html.body.findAll(name = "div", recursive = False):
+                self.__delete_div(div)
+
+
     def __delete_before(self, html):
         if self.__delete_before_date:
             divs = filter(lambda div: \
@@ -765,11 +776,11 @@ class Feed:
         result = 0
         div = None
         try:
-            self.__report("downloading", self.__url)
+            self.__report("downloading", self.__url, 1)
             page = urlopener.open(quote_url_path(self.__url))
             try:
                 self._real_url = page.geturl()
-                self.__report("parsing", self.__url)
+                self.__report("parsing", self.__url, 10)
                 xml = BeautifulStoneSoup(page, markupMassage = massage)
             finally:
                 page.close()
@@ -781,6 +792,9 @@ class Feed:
             if os.path.exists(path):
                 f = open(path, "r")
                 html = BeautifulSoup(f)
+                if self.__always_clear_cache:
+                    self.__report("deleting", "", 15)
+                    self.__delete_all_divs(html)
                 f.close()
             else:
                 html = BeautifulSoup(self.__page_template)
@@ -794,11 +808,21 @@ class Feed:
             div.append(NavigableString(update_str))
             div.append(Tag(html, "hr"))
             get_or_create_tag(html, ["body"]).insert(0, div)
-            [self.__add_item(div, tag) for tag in xml.findAll(["item", "entry"])]
+            to_process = []
+            for item in xml.findAll(["item", "entry"]):
+                link = self.__get_item_link(item)
+                if self.__match_link(link) and not self.__outdated(item) \
+                    and not self.__item_exists(div.parent, link):
+                    to_process.append(item)
+            count = len(to_process)
+            for i in range(count):
+                self.__add_item(div, to_process[i], 20 + (i * 60 / count))
             result = self.__get_new_item_count(div)
             if result:
-                self.__delete_before(html)
-                self.__report("downloading images", "")
+                if not self.__always_clear_cache:
+                    self.__report("deleting", "", 80)
+                    self.__delete_before(html)
+                self.__report("downloading images", "", 90)
                 self.__downloader.wait_completion()
                 f = open(path, "w")
                 f.write(html.prettify())
@@ -821,7 +845,5 @@ class Feed:
             f = open(path, "r")
             html = BeautifulSoup(f)
             f.close()
-            if html.body:
-                for div in html.body.findAll(name = "div", recursive = False):
-                    self.__delete_div(div)
+            self.__delete_all_divs(html)
             os.remove(path)
