@@ -25,7 +25,7 @@ gtk.gdk.threads_init()
 #This is to be able to start Feed Circuit on PC
 try:
     import osso
-    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.9", False)
+    osso_context = osso.Context("org.maemo.feedcircuit", "0.7.0", False)
 except ImportError:
     pass
 
@@ -150,7 +150,7 @@ class FeedConfig:
     def __init__(self, cache_path, url, name, mode, days_to_keep,
         include, exclude, ignore_print_version, icon_filename,
         message_count, as_is, content_tag, print_version_link, reformatter_url,
-        next_page_link, id = None):
+        next_page_link, checked, id = None):
         self.id = id
         if not self.id:
             self.id = uuid.uuid1().hex
@@ -172,7 +172,8 @@ class FeedConfig:
         self.icon = None
         self.iter = None
         self.ord_num = 0
-        self.checked = False
+        self.checked = checked
+        self.progress = 0
 
 
     def get_icon(self):
@@ -229,7 +230,8 @@ class FeedListModel(gtk.GenericTreeModel):
 
     def __init__(self, config_path):
         gtk.GenericTreeModel.__init__(self)
-        self.column_types = (gtk.gdk.Pixbuf, str, str, gobject.TYPE_BOOLEAN)
+        self.column_types = (gtk.gdk.Pixbuf, str, str, gobject.TYPE_BOOLEAN,
+            str, gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN, int)
         self.list = []
         self.map = {}
         self.config_path = config_path
@@ -268,6 +270,14 @@ class FeedListModel(gtk.GenericTreeModel):
                 return self.list[self.map[rowref]].get_time()
             elif column is 3:
                 return self.list[self.map[rowref]].checked
+            elif column is 4:
+                return self.list[self.map[rowref]].name
+            elif column is 5:
+                return self.list[self.map[rowref]].progress == 0
+            elif column is 6:
+                return self.list[self.map[rowref]].progress != 0
+            elif column is 7:
+                return self.list[self.map[rowref]].progress
 
 
     def on_iter_next(self, rowref):
@@ -312,6 +322,8 @@ class FeedListModel(gtk.GenericTreeModel):
         parser.read([self.get_config_file_name()])
         self.cache_path = parser.get_default("global", "cache path", self.config_path)
         self.use_checkboxes = parser.getboolean_default("global", "use checkboxes", False)
+        self.use_new_window = parser.getboolean_default("global", "use new window", False)
+        self.show_banner = parser.getboolean_default("global", "show banner", False)
         for section in parser.sections():
             if section != "global":
                 feed_config = FeedConfig(
@@ -330,6 +342,7 @@ class FeedListModel(gtk.GenericTreeModel):
                     parser.get_default(section, "print version link", ""),
                     parser.get_default(section, "reformatter url", ""),
                     parser.get_default(section, "next page link", ""),
+                    parser.getboolean_default(section, "checked", False),
                     section)
                 feed_config.ord_num = parser.getint_default(section, "order number", 0)
                 self.list.append(feed_config)
@@ -344,6 +357,8 @@ class FeedListModel(gtk.GenericTreeModel):
             parser.add_section("global")
             parser.set("global", "cache path", self.cache_path)
             parser.set("global", "use checkboxes", str(self.use_checkboxes))
+            parser.set("global", "use new window", str(self.use_new_window))
+            parser.set("global", "show banner", str(self.show_banner))
             i = 0
             for feed_config in self.list:
                 parser.add_section(feed_config.id)
@@ -361,6 +376,7 @@ class FeedListModel(gtk.GenericTreeModel):
                 parser.set(feed_config.id, "print version link", feed_config.print_version_link)
                 parser.set(feed_config.id, "reformatter url", feed_config.reformatter_url)
                 parser.set(feed_config.id, "next page link", feed_config.next_page_link)
+                parser.set(feed_config.id, "checked", str(feed_config.checked))
                 parser.set(feed_config.id, "order number", str(i))
                 i += 1
             parser.write(f)
@@ -379,6 +395,11 @@ class FeedListModel(gtk.GenericTreeModel):
         self.save_config()
 
 
+    def refresh(self, rowref):
+        index = self.map[rowref]
+        self.row_changed((index,), self.get_iter((index,)))
+
+
     def delete(self, rowref):
         index = self.map[rowref]
         del self.list[index]
@@ -425,6 +446,7 @@ class FeedListModel(gtk.GenericTreeModel):
         feed_config = self.list[path[0]]
         feed_config.checked = not feed_config.checked
         self.row_changed(path, self.get_iter(path))
+        self.save_config()
 
 
     def set_checkboxes(self, value):
@@ -472,14 +494,19 @@ class PreferencesWindow(gtk.Dialog):
         self.choose_button.connect("clicked", self.choose_button_clicked)
         hbox.pack_start(self.choose_button, expand = False)
         self.checkboxes_check = gtk.CheckButton()
-        table = gtk.Table(rows = 2, columns = 2)
-        texts = [_("Cache path"), _("Show checkboxes")]
+        self.new_win_check = gtk.CheckButton()
+        self.show_banner_check = gtk.CheckButton()
+        table = gtk.Table(rows = 4, columns = 2)
+        texts = [_("Cache path"), _("Show checkboxes"),
+            _("Open in new window"), _("Show update banner")]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
             label.set_alignment(1, 0.5)
             table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
         table.attach(hbox, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
         table.attach(self.checkboxes_check, 1, 2, 1, 2, xpadding = 3, ypadding = 3)
+        table.attach(self.new_win_check, 1, 2, 2, 3, xpadding = 3, ypadding = 3)
+        table.attach(self.show_banner_check, 1, 2, 3, 4, xpadding = 3, ypadding = 3)
         self.vbox.add(table)
         self.show_all()
 
@@ -512,7 +539,7 @@ class ImportWindow(gtk.Dialog):
         self.stock_check = gtk.CheckButton()
         self.stock_check.connect("toggled", self.stock_check_changed)
         table = gtk.Table(rows = 2, columns = 2)
-        texts = [_("From default reader"), _("OPML file")]
+        texts = [_("From default reader"), _("From OPML file")]
         for i in range(0, len(texts)):
             label = gtk.Label(texts[i])
             label.set_alignment(1, 0.5)
@@ -566,7 +593,7 @@ class ExtraWindow(gtk.Dialog):
             (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
              gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
         self.set_title(_("Extra feed settings"))
-        self.days_editor = hildon.NumberEditor(1, 365)
+        self.days_editor = hildon.NumberEditor(0, 365)
         self.include_entry = gtk.Entry()
         self.include_entry.set_width_chars(24)
         self.exclude_entry = gtk.Entry()
@@ -789,17 +816,11 @@ class Program(hildon.Program):
         self.append_action(menu, self.__toolbar, None, gtk.STOCK_DELETE, self.remove)
         menu.append(gtk.SeparatorMenuItem())
         self.append_action(menu, None, _("Import"), None, self.import_feeds)
+        self.append_action(menu, None, _("Export"), None, self.export_feeds)
         self.append_action(menu, self.__toolbar, None, gtk.STOCK_PREFERENCES, self.preferences)
         menu.append(gtk.SeparatorMenuItem())
         self.append_action(menu, None, None, gtk.STOCK_ABOUT, self.about)
         self.append_action(menu, None, None, gtk.STOCK_QUIT, self.stop)
-        #self.__progress_label = gtk.Label()
-        #self.__progress_label.set_alignment(0, 0.5)
-        #self.__progress_label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
-        #status_item = gtk.ToolItem()
-        #status_item.add(self.__progress_label)
-        #status_item.set_expand(True)
-        #self.__toolbar.insert(status_item, -1)
         self.__abort_button = gtk.ToolButton(gtk.STOCK_STOP)
         self.__abort_button.connect("clicked", self.abort)
         self.__abort_button.set_no_show_all(True)
@@ -833,6 +854,16 @@ class Program(hildon.Program):
         render_text.set_property("ellipsize", pango.ELLIPSIZE_MIDDLE)
         col.pack_start(render_text, expand=True)
         col.add_attribute(render_text, 'markup', 1)
+        col.add_attribute(render_text, 'visible', 5)
+        render_progress = gtk.CellRendererProgress()
+        try:
+            render_progress.set_property("text-xalign", 0)
+        except:
+            pass # text-xalign doesn't exist before GTK+ 2.12
+        col.pack_start(render_progress, expand=True)
+        col.add_attribute(render_progress, 'text', 4)
+        col.add_attribute(render_progress, 'visible', 6)
+        col.add_attribute(render_progress, 'value', 7)
         col.set_expand(True)
         self.__view.append_column(col)
         
@@ -929,7 +960,10 @@ class Program(hildon.Program):
     def open_feed(self, feed_config):
         path = os.path.join(self.__model.cache_path, feed_config.id + ".html")
         if os.path.exists(path):
-            webbrowser.open(path)
+            new_win = 0
+            if self.__model.use_new_window:
+                new_win = 2
+            webbrowser.open(path, new_win)
             feed_config.message_count = 0
             self.__model.update(feed_config.id)
         else:
@@ -940,6 +974,14 @@ class Program(hildon.Program):
             msg_dlg.destroy()
 
 
+    def set_banner_text(self, text):
+        if self.__progress_banner:
+            self.__progress_banner.set_text(text)
+        else:
+            self.__progress_banner = \
+                hildon.hildon_banner_show_animation(self.__window, "qgn_indi_pball_a", text)
+
+
     def get_selected_feeds(self):
         if self.__model.use_checkboxes:
             return filter(lambda fc: fc.checked, self.__model.list)
@@ -964,40 +1006,41 @@ class Program(hildon.Program):
         self.open_feed(self.__model.list[path[0]])
 
 
-    def feed_updated(self, feed_config, new_item_count):
-        feed_config.message_count += new_item_count
-        self.__model.update(feed_config.id)
-
-
     def abort(self, source):
         self.__abort_lock.acquire()
         self.__aborted = True
         self.__abort_lock.release()
-        if self.__thread:
-            self.__thread.join()
+        self.__abort_button.set_sensitive(False)
+        self.set_banner_text(_("aborting"))
 
 
     def update_started(self):
+        self.__abort_button.set_sensitive(True)
         self.__abort_button.show()
 
 
+    def update_status(self, feed_config, action, item, i, feed_count, percent, j, url_count):
+        if percent:
+            feed_config.progress = (j * 100 + percent) / url_count
+            self.__model.refresh(feed_config.id)
+        if self.__model.show_banner:
+            self.set_banner_text( \
+                "(%u/%u) %s %s" % (i, feed_count, _(action), item))
+
+
+    def update_feed_complete(self, feed_config, new_item_count):
+        feed_config.message_count += new_item_count
+        feed_config.progress = 0
+        self.__model.update(feed_config.id)
+
+
     def update_complete(self):
-        #self.update_status("")
         if self.__progress_banner:
             self.__progress_banner.destroy()
             self.__progress_banner = None
         self.__abort_button.hide()
 
 
-    def update_status(self, text):
-        if self.__progress_banner:
-            self.__progress_banner.set_text(text)
-        else:
-            self.__progress_banner = \
-                hildon.hildon_banner_show_animation(self.__window, "qgn_indi_pball_a", text)
-        #self.__progress_label.set_text(text)
-
-
     def callback(self, item):
         self.__abort_lock.acquire()
         abort = self.__aborted
@@ -1005,9 +1048,9 @@ class Program(hildon.Program):
         self.__abort_lock.release()
         if abort:
             sys.exit()
-        text = "(" + str(self.__progress_info[0]) + "/" + str(self.__progress_info[1]) + ") " + \
-            _(str(item[0])) + " " + str(item[1])
-        gobject.idle_add(self.update_status, text)
+        gobject.idle_add(self.update_status, self.__progress_info[0], item[0], item[1], \
+            self.__progress_info[1], self.__progress_info[2], item[2], \
+            self.__progress_info[3], self.__progress_info[4])
 
 
     def update_feeds(self, feed_configs):
@@ -1020,25 +1063,37 @@ class Program(hildon.Program):
             i = 0
             for feed_config in feed_configs:
                 i += 1
-                self.__progress_info = (i, len(feed_configs))
-                urls = feed_config.get_urls()
                 new_item_count = 0
+                urls = feed_config.get_urls()
+                url_count = len(urls)
+                feed_count = len(feed_configs)
                 title = None
-                if len(urls) == 1:
+                if url_count == 1:
                     title = feed_config.name
-                for url in urls:
-                    feed = feedcircuit.Feed(url = url, path = self.__model.cache_path, cache_path = ".cache",
-                        title = title, filename = feed_config.id + ".html",
-                        callback = self.callback, try_use_print_version = not feed_config.ignore_print_version,
-                        delete_before_date = date.today() - timedelta(feed_config.days_to_keep),
-                        inline_items = feed_config.mode == FeedConfig.mode_inline,
-                        cache_items = feed_config.mode == FeedConfig.mode_cache,
-                        include = feed_config.include, exclude = feed_config.exclude,
-                        as_is = feed_config.as_is, content_kwargs = feed_config.get_content_kwargs(),
-                        print_version_keywords = feed_config.get_print_version_keywords(),
-                        reformatter_url = feed_config.reformatter_url, next_page_keyword = feed_config.next_page_link)
-                    new_item_count += feed.update()
-                gobject.idle_add(self.feed_updated, feed_config, new_item_count)
+                j = 0
+                try:
+                    for url in urls:
+                        self.__progress_info = (feed_config, i, feed_count, j, url_count)
+                        del_before = None
+                        clear_cache = True
+                        if feed_config.days_to_keep:
+                            del_before = date.today() - timedelta(feed_config.days_to_keep)
+                            clear_cache = False
+                        feed = feedcircuit.Feed(url = url, path = self.__model.cache_path, cache_path = ".cache",
+                            title = title, filename = feed_config.id + ".html",
+                            callback = self.callback, try_use_print_version = not feed_config.ignore_print_version,
+                            delete_before_date = del_before, always_clear_cache = clear_cache,
+                            inline_items = feed_config.mode == FeedConfig.mode_inline,
+                            cache_items = feed_config.mode == FeedConfig.mode_cache,
+                            include = feed_config.include, exclude = feed_config.exclude,
+                            as_is = feed_config.as_is, content_kwargs = feed_config.get_content_kwargs(),
+                            print_version_keywords = feed_config.get_print_version_keywords(),
+                            reformatter_url = feed_config.reformatter_url, next_page_keyword = feed_config.next_page_link,
+                            new_window = self.__model.use_new_window)
+                        new_item_count += feed.update()
+                        j += 1
+                finally:
+                    gobject.idle_add(self.update_feed_complete, feed_config, new_item_count)
         finally:
             gobject.idle_add(self.update_complete)
             self.__update_lock.release()
@@ -1094,7 +1149,7 @@ class Program(hildon.Program):
                 msg_dlg.run()
                 msg_dlg.destroy()
             feed_config = FeedConfig(self.__model.cache_path, url, title, FeedConfig.mode_ommit, 7, "", "", False, icon_filename, 0,
-                False, "", "", "", "")
+                False, "", "", "", "", False)
             edit_window = EditWindow(self.__window, feed_config)
             if edit_window.run() == gtk.RESPONSE_ACCEPT:
                 edit_window.update_feed()
@@ -1127,7 +1182,9 @@ class Program(hildon.Program):
                     feed_config.message_count = 0
                     feed = feedcircuit.Feed(None, path = self.__model.cache_path,
                         cache_path = ".cache", filename = feed_config.id + ".html")
-                    feed.delete()
+                    delete_thread = BannerThread(self.__window, _("deleting cache"))
+                    delete_thread.execute(feed.delete)
+                    self.__model.refresh(feed_config.id)
                     if not only_cache:
                         if feed_config.icon_filename:
                             icon_filepath = os.path.join(self.__model.cache_path, feed_config.icon_filename)
@@ -1143,7 +1200,9 @@ class Program(hildon.Program):
             filename = os.path.split(file)[1]
             if filename != "config":
                 shutil.move(file, os.path.join(to_path, filename))
-        shutil.move(os.path.join(from_path, ".cache"), os.path.join(to_path, ".cache"))
+        dot_cache_path = os.path.join(from_path, ".cache")
+        if os.path.exists(dot_cache_path):
+            shutil.move(dot_cache_path, os.path.join(to_path, ".cache"))
 
 
     def import_opml(self, filename = None):
@@ -1183,7 +1242,7 @@ class Program(hildon.Program):
                         msg_dlg.destroy()
                 feed_config = FeedConfig(self.__model.cache_path, outline["xmlurl"], outline["title"],
                     FeedConfig.mode_ommit, 7, "", "", False, icon_file_name, 0,
-                    False, "", "", "", "")
+                    False, "", "", "", "", False)
                 self.__model.append(feed_config)
         finally:
             f.close()
@@ -1210,17 +1269,49 @@ class Program(hildon.Program):
             imp_window.destroy()
 
 
+    def export_feeds(self, source):
+        dlg = gtk.FileChooserDialog(title = _("Select opml file"), \
+            parent = self.__window, action = gtk.FILE_CHOOSER_ACTION_SAVE,
+            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+        if dlg.run() == gtk.RESPONSE_ACCEPT:
+            filename = dlg.get_filename()
+            if not filename.endswith(".opml"):
+                filename += ".opml"
+            f = open(filename, "w")
+            try:
+                f.write('<?xml version="1.0" encoding="utf-8"?>\n')
+                f.write('<opml version="1.0">\n')
+                f.write('<head/>\n')
+                f.write('<body>\n')
+                for feed_config in self.__model.list:
+                    title = feed_config.name.replace("<", "&lt;").replace(">", "&gt;")
+                    for url in feed_config.url.split(" "):
+                        f.write('<outline title="%s" description="%s" xmlUrl="%s" /> \n' % \
+                            (title, title, url))
+                f.write('</body>\n')
+                f.write('</opml>\n')
+            finally:
+                f.close()
+        dlg.destroy()
+        
+
+
     def preferences(self, source):
         pref_window = PreferencesWindow(self.__window)
         try:
             pref_window.path_entry.set_text(self.__model.cache_path)
             pref_window.checkboxes_check.set_active(self.__model.use_checkboxes)
+            pref_window.new_win_check.set_active(self.__model.use_new_window)
+            pref_window.show_banner_check.set_active(self.__model.show_banner)
             if pref_window.run() == gtk.RESPONSE_ACCEPT:
                 new_path = os.path.abspath(pref_window.path_entry.get_text())
                 if new_path != self.__model.cache_path:
-                    self.move_cache(self.__model.cache_path, new_path)
+                    move_thread = BannerThread(self.__window, _("moving cache"))
+                    move_thread.execute(self.move_cache, self.__model.cache_path, new_path)
                     self.__model.update_cache_path(new_path)
                 self.__model.set_checkboxes(pref_window.checkboxes_check.get_active())
+                self.__model.use_new_window = pref_window.new_win_check.get_active()
+                self.__model.show_banner = pref_window.show_banner_check.get_active()
                 self.check_col.set_visible(self.__model.use_checkboxes)
                 self.refresh_selected_item.set_property("visible", 
                     not self.__model.use_checkboxes)
@@ -1238,7 +1329,7 @@ class Program(hildon.Program):
     def about(self, source):
         dialog = gtk.AboutDialog()
         dialog.set_name("FeedCircuit")
-        dialog.set_version("0.6.9")
+        dialog.set_version("0.7.0")
         dialog.set_website("http://feedcircuit.garage.maemo.org")
         dialog.set_authors([_("George Kibardin")])
         dialog.run()
